@@ -7,11 +7,9 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id$
-###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '4.25'; # also change in Devel::NYTProf::Core
+our $VERSION = '5.06'; # also change in Devel::NYTProf::Core
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -182,7 +180,7 @@ statement executed in foo()>! Here's another example:
   }
 
 After the first time around the loop, any further time spent evaluating the
-condition (waiting for input in this example) would be be recorded as having
+condition (waiting for input in this example) would be recorded as having
 been spent I<on the last statement executed in the loop>! (Until perl bug
 #60954 is fixed this problem still applies to some loops. For more information
 see L<http://rt.perl.org/rt3/Ticket/Display.html?id=60954>)
@@ -204,8 +202,8 @@ For each subroutine called, separate counts and durations are stored I<for each
 location that called the subroutine>.
 
 Subroutine entry is detected by intercepting the C<entersub> opcode. Subroutine
-exit is detected via perl's internal save stack. The result is both extremely
-fast and very robust.
+exit is detected via perl's internal save stack. As a result the subroutine
+profiler is both fast and robust.
 
 =head3 Subroutine Recursion
 
@@ -310,6 +308,12 @@ a backslash.
 Append the current process id to the end of the filename.
 
 This avoids concurrent, or consecutive, processes from overwriting the same file.
+If a fork is detected during profiling then the child process will automatically
+add the process id to the filename.
+
+=head2 addtimestamp=1
+
+Append the current time, as integer epoch seconds, to the end of the filename.
 
 =head2 trace=N
 
@@ -370,13 +374,12 @@ If you find any other examples of the effect of optimizer on NYTProf output
 
 Set to 0 to disable the collection of subroutine caller and timing details.
 
-=head2 blocks=0
+=head2 blocks=1
 
-Set to 0 to disable the determination of block and subroutine location per statement.
-This makes the profiler about 50% faster (as of July 2008) and produces smaller
-output files, but you lose some valuable information. The extra cost is likely
-to be reduced in later versions anyway, as little optimization has been done on
-that part of the code.
+Set to 1 to enable the determination of block and subroutine location per statement.
+This makes the profiler about 50% slower (as of July 2008) and produces larger
+output files, but you gain some valuable insight in where time is spent in the
+blocks within large subroutines and scripts.
 
 =head2 stmts=0
 
@@ -387,6 +390,25 @@ This significantly reduces the overhead of the profiler and can also be useful
 for profiling large applications that would normally generate a very large
 profile data file.
 
+=head2 calls=N
+
+This option is I<new and experimental>.
+
+With calls=1 (the default) subroutine call I<return> events are emitted into
+the data stream as they happen.  With calls=2 subroutine call I<entry> events
+are also emitted. With calls=0 no subroutine call events are produced.
+This option depends on the C<subs> option being enabled, which it is by default.
+
+The L<nytprofcalls> utility can be used to process this data. It too is I<new
+and experimental> and so likely to change.
+
+The subroutine profiler normally gathers data in memory and outputs a summary
+when the profile data is being finalized, usually when the program has finished.
+The summary contains aggregate information for all the calls from one location
+to another, but the details of individual calls have been lost.
+The calls option enables the recording of individual call events and thus
+more detailed analysis and reporting of that data.
+
 =head2 leave=0
 
 Set to 0 to disable the extra work done by the statement profiler
@@ -519,6 +541,19 @@ When using the C<subs=0> option to disable the subroutine profiler the
 C<posix_exit> option can be used to tell NYTProf to take other steps to arrange
 for C<DB::finish_profile()> to be called before C<POSIX::_exit()>.
 
+=head2 libcexit=1
+
+Arranges for L</finish_profile> to be called via the C library C<atexit()> function.
+This may help some tricky cases where the process may exit without perl
+executing the C<END> block that NYTProf uses to call /finish_profile().
+
+=head2 endatexit=1
+
+Sets the PERL_EXIT_DESTRUCT_END flag in the PL_exit_flags of the perl interpreter.
+This makes perl run C<END> blocks in perl_destruct() instead of perl_run()
+which may help in cases, like Apache, where perl is embedded but perl_run()
+isn't called.
+
 =head2 forkdepth=N
 
 When a perl process that is being profiled executes a fork() the child process
@@ -568,7 +603,9 @@ to stop collecting profile data, and calling DB::enable_profile() to start
 collecting profile data.
 
 Using the C<start=no> option lets you leave the profiler disabled initially
-until you call DB::enable_profile() at the right moment.
+until you call DB::enable_profile() at the right moment. You still need to
+load Devel::NYTProf as early as possible, even if you don't call
+DB::enable_profile() until much later. See also L</use_db_sub=1>.
 
 The profile output file can't be used until it's been properly completed and
 closed.  Calling DB::disable_profile() doesn't do that.  To make a profile file
@@ -768,20 +805,20 @@ You can reduce the cost of profiling by adjusting some options. The trade-off
 is reduced detail and/or accuracy in reports.
 
 If you don't need statement-level profiling then you can disable it via L</stmts=0>.
-If you do want it but don't mind loosing block-level timings then set L</blocks=0>.
 To further boost statement-level profiling performance try L</leave=0> but note that
 I<will> apportion timings for some kinds of statements less accurate).
 
+If you don't need call stacks or flamegraph then disable it via L</calls=0>.
 If you don't need subroutine profiling then you can disable it via L</subs=0>.
 If you do need it but don't need timings for perl opcodes then set L</slowops=0>.
 
-Generally speaking, setting blocks=0 and slowops=0 will give you a useful boost
+Generally speaking, setting calls=0 and slowops=0 will give you a useful boost
 with the least loss of detail.
 
 Another approach is to only enable NYTProf in the sections of code that
 interest you. See L</RUN-TIME CONTROL OF PROFILING> for more details.
 
-To speed up L<nytprofhtml> try using the --minimal (-m) option.
+To speed up L<nytprofhtml> try using the --minimal (-m) or --no-flame options.
 
 =head1 REPORTS
 
@@ -801,15 +838,15 @@ Creates attractive, richly annotated, and fully cross-linked html
 reports (including statistics, source code and color highlighting).
 This is the main report generation tool for NYTProf.
 
-=head2 nytprofcsv
-
-Creates comma delimited profile reports. Old and limited.
-
 =head2 nytprofcg
 
 Translates a profile into a format that can be loaded into KCachegrind
 L<http://kcachegrind.sourceforge.net>
 
+=head2 nytprofcalls
+
+Reads a profile and processes the calls events it contains.
+
 =head2 nytprofmerge
 
 Reads multiple profile data files and writes out a new file containing the merged profile data.
@@ -981,6 +1018,10 @@ Why 'realtime' can appear to go backwards:
 
   http://preview.tinyurl.com/5wawnn
 
+The PostgreSQL pg_test_timing utility documentation has a good summary of timing issues:
+
+  http://www.postgresql.org/docs/9.2/static/pgtesttiming.html
+
 =for comment
 http://preview.tinyurl.com/5wawnn redirects to:
 http://groups.google.com/group/comp.os.linux.development.apps/tree/browse_frm/thread/dc29071f2417f75f/ac44671fdb35f6db?rnum=1&_done=%2Fgroup%2Fcomp.os.linux.development.apps%2Fbrowse_frm%2Fthread%2Fdc29071f2417f75f%2Fc46264dba0863463%3Flnk%3Dst%26rnum%3D1%26
@@ -1039,6 +1080,8 @@ impacted by those more than it otherwise would.
 
 =head3 Windows
 
+B<THIS SECTION DOESN'T MATCH THE CODE>
+
 On Windows NYTProf uses Time::HiRes which uses the windows
 QueryPerformanceCounter() API with some extra logic to adjust for the current
 clock speed and try to resync the raw counter to wallclock time every so often
@@ -1092,6 +1135,10 @@ it's very likely to be deprecated in a future release).
 L<Devel::NYTProf::ReadStream> is the module that lets you read a profile data
 file as a stream of chunks of data.
 
+Other tools:
+
+DTrace L<https://speakerdeck.com/mrallen1/perl-dtrace-and-you>
+
 =head1 TROUBLESHOOTING
 
 =head2 "Profile data incomplete, ..." or "File format error: ..."
@@ -1186,7 +1233,7 @@ For more details see L</HISTORY> below.
 =head1 COPYRIGHT AND LICENSE
 
   Copyright (C) 2008 by Adam Kaplan and The New York Times Company.
-  Copyright (C) 2008-2010 by Tim Bunce, Ireland.
+  Copyright (C) 2008-2013 by Tim Bunce, Ireland.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
