@@ -9,6 +9,7 @@ use ExtUtils::testlib;
 use Getopt::Long;
 use Test::More;
 use Data::Dumper;
+use File::Spec;
 use File::Temp qw(tempfile);
 use List::Util qw(shuffle);
 
@@ -34,14 +35,14 @@ my %opts = (
     html         => $ENV{NYTPROF_TEST_HTML},
     mergerdt     => $ENV{NYTPROF_TEST_MERGERDT}, # overkill, but handy
 );
-GetOptions(\%opts, qw/p=s I=s v|verbose d|debug html open profperlopts=s leave=i use_db_sub=i savesrc=i compress=i one abort/)
+GetOptions(\%opts, qw/p=s I=s v|verbose d|debug html open profperlopts=s blocks=i leave=i use_db_sub=i savesrc=i compress=i one abort/)
     or exit 1;
 
 $opts{v}    ||= $opts{d};
 $opts{html} ||= $opts{open};
 
 # note some env vars that might impact the tests
-$ENV{$_} && warn "$_=$ENV{$_}\n" for qw(PERL5DB PERL5OPT PERL_UNICODE PERLIO);
+$ENV{$_} && warn "$_='$ENV{$_}'\n" for qw(PERL5DB PERL5OPT PERL_UNICODE PERLIO);
 
 if ($ENV{NYTPROF}) {                        # avoid external interference
     warn "Existing NYTPROF env var value ($ENV{NYTPROF}) ignored for tests. Use NYTPROF_TEST env var if need be.\n";
@@ -60,10 +61,12 @@ for my $opt (qw(trace)) {
 }
 
 
-my $tests_per_extn = {
-    p   => 1,
-    rdt => ($opts{mergerdt}) ? 2 : 1,
-    x   => 3
+my $text_extn_info = {
+    p     => { order => 10, tests => 1, },
+    rdt   => { order => 20, tests => ($opts{mergerdt}) ? 2 : 1, },
+    x     => { order => 30, tests => 3, },
+    calls => { order => 40, tests => 1, },
+    pf    => { order => 50, tests => 2, },
 };
 
 chdir('t') if -d 't';
@@ -77,9 +80,11 @@ my $bindir      = (grep {-d} qw(./blib/script ../blib/script))[0] || do {
     warn "Couldn't find blib/script directory, so using $bin";
     $bin;
 };
-my $nytprofcsv  = "$bindir/nytprofcsv";
-my $nytprofhtml = "$bindir/nytprofhtml";
-my $nytprofmerge= "$bindir/nytprofmerge";
+my $nytprofcsv   = File::Spec->catfile($bindir, "nytprofcsv");
+my $nytprofcalls = File::Spec->catfile($bindir, "nytprofcalls");
+my $nytprofhtml  = File::Spec->catfile($bindir, "nytprofhtml");
+my $nytprofpf    = File::Spec->catfile($bindir, "nytprofpf");
+my $nytprofmerge = File::Spec->catfile($bindir, "nytprofmerge");
 
 my $path_sep = $Config{path_sep} || ':';
 my $perl5lib = $opts{I} || join($path_sep, @INC);
@@ -94,41 +99,51 @@ if ($opts{one}) {           # for one quick test
     $opts{use_db_sub} = 0;
     $opts{savesrc}    = 1;
     $opts{compress}   = 1;
+    $opts{calls}      = 2;
+    $opts{blocks}     = 1;
 }
 
 # force savesrc off for perl 5.11.2 due to perl bug RT#70804
 $opts{savesrc} = 0 if $] eq "5.011002";
 
+my @test_opt_blocks     = (defined $opts{blocks})     ? ($opts{blocks})     : (1);
 my @test_opt_leave      = (defined $opts{leave})      ? ($opts{leave})      : (0, 1);
 my @test_opt_use_db_sub = (defined $opts{use_db_sub}) ? ($opts{use_db_sub}) : (0, 1);
 my @test_opt_savesrc    = (defined $opts{savesrc})    ? ($opts{savesrc})    : (0, 1);
 my @test_opt_compress   = (defined $opts{compress})   ? ($opts{compress})   : (0, 1);
+my @test_opt_calls      = (defined $opts{calls})      ? ($opts{calls})      : (0, 1, 2);
 
 sub mk_opt_combinations {
     my ($overrides) = @_;
 
     my @opt_combinations;
     my %seen;
+
+    for my $blocks (@test_opt_blocks) {
     for my $leave (@test_opt_leave) {
-        for my $use_db_sub (@test_opt_use_db_sub) {
-            for my $savesrc (@test_opt_savesrc) {
-                for my $compress (@test_opt_compress) {
-                    my $o = {
-                        start      => 'init',
-                        slowops    => 2,
-                        leave      => $leave,
-                        use_db_sub => $use_db_sub,
-                        savesrc    => $savesrc,
-                        compress   => $compress,
-                        ($overrides) ? %$overrides : (),
-                    };
-                    my $key = join "\t", map { "$_=>$o->{$_}" } sort keys %$o;
-                    next if $seen{$key}++;
-                    push @opt_combinations, $o;
-                }
-            }
-        }
-    }
+    for my $use_db_sub (@test_opt_use_db_sub) {
+    for my $savesrc (@test_opt_savesrc) {
+    for my $compress (@test_opt_compress) {
+
+            my $o = {
+                start      => 'init',
+                slowops    => 2,
+                blocks     => $blocks,
+                leave      => $leave,
+                use_db_sub => $use_db_sub,
+                savesrc    => $savesrc,
+                compress   => $compress,
+                # we don't need to test the 'calls' opt with all other combinations
+                # so we fudge it here to be on most, but not all, of the time
+                calls      => (!!$savesrc + !!$compress), # 0|1|2
+                ($overrides) ? %$overrides : (),
+            };
+            my $key = join "\t", map { "$_=>$o->{$_}" } sort keys %$o;
+            next if $seen{$key}++;
+            push @opt_combinations, $o;
+
+    } } } } }
+
     @opt_combinations = shuffle @opt_combinations;
     return \@opt_combinations;
 }
@@ -165,7 +180,7 @@ sub do_foreach_opt_combination {
         # did any tests fail?
         my $failed = (count_of_failed_tests() - $prev_failures) ? 1 : 0;
         # record what env settings may have influenced the failure
-        ++$env_influence{$_}{$env->{$_}}{$failed ? 'fail' : 'pass'}
+        ++$env_influence{$_}{$env->{$_}}{$failed ? 'FAIL' : 'pass'}
             for keys %$env;
         $env_failed{ $ENV{NYTPROF} } = $failed;
     }
@@ -181,15 +196,21 @@ sub report_env_influence {
     my @env_influence;
     for my $envvar (sort keys %env_influence) {
         my $variants = $env_influence{$envvar};
+
         local $Data::Dumper::Indent   = 0;
         local $Data::Dumper::Sortkeys = 1;
         local $Data::Dumper::Terse    = 1;
         local $Data::Dumper::Quotekeys= 0;
         local $Data::Dumper::Pair     = ' ';
         $variants->{$_} = Dumper($variants->{$_}) for keys %$variants;
+
+        # was there at least one failure?
+        next unless grep { /FAIL/ } values %$variants;
+
         my $v = (values %$variants)[0]; # use one as a reference
         # all the same?
         next if keys %$variants == grep { $_ eq $v } values %$variants;
+
         push @env_influence, sprintf "%15s: %s\n", $envvar,
             join ', ', map { "$_ => $variants->{$_}" } sort keys %$variants;
     }
@@ -229,14 +250,17 @@ sub run_test_group {
         croak "Can't determine test group";
     }
 
-    my @tests = grep { -f $_ } map { "$group.$_" } sort keys %$tests_per_extn;
-    unlink <$group.*_new{,p}>; # delete _new and _newp files from previous run
+    my @tests = grep { -f $_ }
+        map { "$group.$_" }
+        sort { $text_extn_info->{$a}{order} <=> $text_extn_info->{$b}{order} }
+        keys %$text_extn_info;
+    unlink <$group.*_new*>; # delete _new* files from previous run
 
     if ($opts{v}) {
         print "tests: @tests\n";
         print "perl: $perl\n";
         print "perl5lib: $perl5lib\n";
-        print "nytprofcvs: $nytprofcsv\n";
+        print "nytprofbin: $bindir\n";
     }
 
     plan skip_all => "No '$group.*' test files and no extra_test_code"
@@ -317,7 +341,7 @@ sub run_test {
 
         if ($opts{mergerdt}) { # run the file through nytprofmerge
             my $merged = "$profile_datafile.merged";
-            my $merge_cmd = "$nytprofmerge -v --out=$merged $test_datafile";
+            my $merge_cmd = "$perl $nytprofmerge -v --out=$merged $test_datafile";
             warn "$merge_cmd\n";
             system($merge_cmd) == 0
                 or die "Error running $merge_cmd\n";
@@ -325,12 +349,23 @@ sub run_test {
             unlink $merged;
         }
     }
+    elsif ($type eq 'calls') {
+        if ($env->{calls}) {
+            verify_calls_report($test, $tag, $test_datafile, $outdir);
+        }
+        else {
+            pass("no calls");
+        }
+    }
     elsif ($type eq 'x') {
         mkdir $outdir or die "mkdir($outdir): $!" unless -d $outdir;
         unlink <$outdir/*>;
 
         verify_csv_report($test, $tag, $test_datafile, $outdir);
     }
+    elsif ($type eq 'pf') {
+        verify_platforms_csv_report($test, $tag, $test_datafile, $outdir);
+    }
     elsif ($type =~ /^(?:pl|pm|new|outdir)$/) {
         # skip; handy for "test.pl t/test01.*"
     }
@@ -376,7 +411,7 @@ sub run_perl_command {
 }
 
 
-sub profile {
+sub profile { # TODO refactor to use run_perl_command()?
     my ($test, $profile_datafile) = @_;
 
     my @perl = perl_command_words(skip_sitecustomize => 1);
@@ -398,16 +433,23 @@ sub verify_data {
     SKIP: {
         skip 'Expected profile data does not have VMS paths', 1
             if $^O eq 'VMS' and $test =~ m/test60|test14/i;
-        $profile->normalize_variables;
+        $profile->normalize_variables(1); # and options
         dump_profile_to_file($profile, $test.'_new', $test.'_newp');
-        my @got      = slurp_file($test.'_new'); chomp @got;
-        my @expected = slurp_file($test);        chomp @expected;
-        is_deeply(\@got, \@expected, "$test match generated profile data for $tag")
-            ? unlink($test.'_new')
-            : diff_files($test, $test.'_new', $test.'_newp');
+        is_file_content_same($test.'_new', $test, "$test match generated profile data for $tag");
     }
 }
 
+sub is_file_content_same {
+    my ($got_file, $exp_file, $testname) = @_;
+
+    my @got = slurp_file($got_file); chomp @got;
+    my @exp = slurp_file($exp_file); chomp @exp;
+
+    is_deeply(\@got, \@exp, $testname)
+        ? unlink($got_file)
+        : diff_files($exp_file, $got_file, $got_file."_patch");
+}
+
 
 sub dump_data_to_file {
     my ($profile, $file) = @_;
@@ -437,12 +479,23 @@ sub dump_profile_to_file {
     return;
 }
 
+
 sub diff_files {
     my ($old_file, $new_file, $newp_file) = @_;
 
     # we don't care if this fails, it's just an aid to debug test failures
-    my @opts = split / /, $ENV{NYTPROF_DIFF_OPTS} || $diff_opts;    # e.g. '-y'
-    system("cmp -s $new_file $newp_file || diff @opts $old_file $new_file 1>&2");
+    # XXX needs to behave better on windows
+    my @opts = split / /, $ENV{NYTPROF_DIFF_OPTS} || $diff_opts; # e.g. '-y'
+    system("diff @opts $old_file $new_file 1>&2");
+}
+
+
+sub verify_calls_report {
+    my ($test, $tag, $profile_datafile, $outdir) = @_;
+    my $got_file = "${test}_new";
+    note "generating $got_file";
+    run_command("$perl $nytprofcalls $profile_datafile -stable --calls > $got_file");
+    is_file_content_same($got_file, $test, "$test match generated calls data for $tag");
 }
 
 
@@ -504,7 +557,8 @@ sub verify_csv_report {
 
         next if m/^# Version/;    # Ignore version numbers
 
-        s/^([0-9.]+),([0-9.]+),([0-9.]+),(.*)$/0,$2,0,$4/o;
+        # we allow negative numbers here re RT#85556
+        s/^(-?[0-9.]+),([0-9.]+),([0-9.]+),(.*)$/0,$2,0,$4/o;
         my $t0  = $1;
         my $c0  = $2;
         my $tc0 = $3;
@@ -541,12 +595,33 @@ sub verify_csv_report {
     chomp @got;
     chomp @expected;
     is_deeply(\@got, \@expected, "$test match generated CSV data for $tag") or do {
-        spit_file($test.'_new', join("\n", @got,''), $test.'_newp');
+        write_out_file($test.'_new', join("\n", @got,''), $test.'_newp');
         diff_files($test, $test.'_new', $test.'_newp');
     };
     is(join("\n", @accuracy_errors), '', "$test times should be reasonable");
 }
 
+sub verify_platforms_csv_report {
+    my ($test, $tag, $profile_datafile, $outdir) = @_;
+    
+    my $outfile = "$outdir/$test.csv";
+
+    my $cmd = "$perl $nytprofpf --file=$profile_datafile --out=$outfile";
+    ok run_command($cmd), "nytprofpf runs ok";
+
+    my $got = slurp_file($outfile);
+        
+    #test if all lines from .pf are contained in result file 
+    #(we can not be sure about the order, so we match each line individually)
+    my $match_result = 1;
+    open (EXPECTED, $test); 
+    while (<EXPECTED>) {
+        $match_result = $match_result && $got =~ m/$_/;
+    }
+    close (EXPECTED);    
+
+    ok $match_result, "$outfile file matches $test";
+}
 
 sub pop_times {
     my $hash = shift || return;
@@ -562,7 +637,7 @@ sub number_of_tests {
     my $total_tests = 0;
     for (@_) {
         next unless m/\.(\w+)$/;
-        my $tests = $tests_per_extn->{$1};
+        my $tests = $text_extn_info->{$1}{tests};
         warn "Unknown test type '$1' for test file '$_'\n" if not defined $tests;
         $total_tests += $tests if $tests;
     }
@@ -579,7 +654,7 @@ sub slurp_file {    # individual lines in list context, entire file in scalar co
 }
 
 
-sub spit_file {
+sub write_out_file {
     my ($file, $content, $rename_existing) = @_;
     rename $file, $rename_existing or warn "rename($file, $rename_existing): $!"
         if $rename_existing && -f $file;
